/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2020 SEGGER Microcontroller GmbH             *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* condition is met:                                                  *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this condition and the following disclaimer.             *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED. IN NO EVENT SHALL SEGGER Microcontroller BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : ARM_Startup.s
Purpose   : Generic startup and exception handlers for ARM devices.

Additional information:
  Preprocessor Definitions
    __FLASH_BUILD
      If defined,
        additional code can be executed in Reset_Handler,
        which might be needed to run from Flash but not to run from RAM.

    __SOFTFP__
      Defined by the build system.
      If not defined, the FPU is enabled for floating point operations.
*/

#if defined(__ARM_ARCH_4T__)
.macro blx reg
  mov lr, pc
  bx \reg
.endm
#endif

        .syntax unified  

/*********************************************************************
*
*       Global data
*
**********************************************************************
*/
/*********************************************************************
*
*  Setup of the exception table
*
*/
        .section .vectors, "ax"
        .code 32
        .balign 4
        .global _vectors
_vectors:
        ldr     PC, =Reset_Handler
        ldr     PC, =undef_handler
        ldr     PC, =swi_handler
        ldr     PC, =pabort_handler
        ldr     PC, =dabort_handler
        nop
        ldr     PC, =irq_handler
        ldr     PC, =fiq_handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       Reset_Handler
*
*  Function description
*    Exception handler for reset.
*    Generic bringup of a Cortex-M system.
*/
        .section .init, "ax"
        .code 32
        .balign 4
        .global reset_handler
        .global Reset_Handler
        .equ reset_handler, Reset_Handler
        .type Reset_Handler, function
Reset_Handler:
  /****************************************************************************
   * TODO: Configure target system.                                           *
   ****************************************************************************/
#ifdef __FLASH_BUILD
  /****************************************************************************
   * TODO: Put any FLASH build configuation specific code here                *
   ****************************************************************************/
#endif
#if !defined(__SOFTFP__)
        //
        // Enable CP11, CP10 and VFP
        //
        mrc     P15, #0x00, R0, C1, C0, #0x02
        orr     R0, R0, #0x00F00000
        mcr     P15, #0x00, R0, C1, C0, #0x02
        mov     R0, #0x40000000
        fmxr    FPEXC, r0
#endif
        //
        // Jump to runtime initialization, which calls main().
        //
        b       _start

/*********************************************************************
*
*       Default exception handlers
*
*  These are declared weak symbols so the can be redefined in user code.
*
**********************************************************************
*/
        .section .init, "ax"
        .code 32
        .balign 4

        .global undef_handler
        .weak undef_handler
        .type undef_handler, function
undef_handler:
        b       undef_handler

        .global swi_handler
        .weak swi_handler
        .type swi_handler, function
swi_handler:
        b       swi_handler
  
        .global pabort_handler
        .weak pabort_handler
        .type pabort_handler, function
pabort_handler:
        b       pabort_handler

        .global dabort_handler
        .weak dabort_handler
        .type dabort_handler, function
dabort_handler:
        b       dabort_handler

        .global irq_handler
        .weak irq_handler
        .type irq_handler, function
irq_handler:
        b       irq_handler

        .global fiq_handler
        .weak fiq_handler
        .type fiq_handler, function
fiq_handler:
        b       fiq_handler

/*************************** End of file ****************************/
